@@ -13,6 +13,9 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
+from __future__ import print_function
+
+import io
 import json
 import re
 import sys
@@ -22,16 +25,19 @@ import random
 import signal
 import socket
 import logging
-from contextlib import contextmanager
+from contextlib import contextmanager, closing
 from optparse import Values
 
 import eventlet
 import eventlet.pools
 from eventlet.green.httplib import CannotSendRequest
 
+import requests.exceptions
+
 import swiftclient as client
 
-from swiftbench.utils import config_true_value, using_http_proxy
+from swiftbench.utils import config_true_value, using_http_proxy, \
+    get_size_bytes
 
 
 try:
@@ -58,7 +64,8 @@ def delete_containers(logger, conf):
     def _deleter(url, token, container):
         try:
             client.delete_container(url, token, container)
-        except client.ClientException as e:
+        except (client.ClientException,
+                requests.exceptions.ConnectionError) as e:
             if e.http_status != HTTP_CONFLICT:
                 logger.warn("Unable to delete container '%s'. "
                             "Got http status '%d'."
@@ -107,12 +114,12 @@ class SourceFile(object):
             raise StopIteration
         chunk_size = min(self.size - self.pos, self.chunk_size)
         self.pos += chunk_size
-        return '0' * chunk_size
+        return b'0' * chunk_size
 
     def read(self, desired_size):
         chunk_size = min(self.size - self.pos, desired_size)
         self.pos += chunk_size
-        return '0' * chunk_size
+        return b'0' * chunk_size
 
 
 class ConnectionPool(eventlet.pools.Pool):
@@ -147,7 +154,8 @@ class BenchServer(object):
         while True:
             client, address = s.accept()
             self.logger.debug('Accepting connection from %s:%s', *address)
-            client_file = client.makefile('rb+', 1)
+            client_file = client.makefile('rwb', 1)
+            client_file = io.TextIOWrapper(client_file)
             json_data = client_file.read()
             conf = Values(json.loads(json_data))
 
@@ -217,14 +225,17 @@ class Bench(object):
             logger.warn("Communication with Swift server is going through "
                         "HTTP proxy server. This could affect test result")
 
-        self.object_size = int(conf.object_size)
+        self.object_size = get_size_bytes(conf.object_size)
         self.object_sources = conf.object_sources
-        self.lower_object_size = int(conf.lower_object_size)
-        self.upper_object_size = int(conf.upper_object_size)
+        self.lower_object_size = get_size_bytes(conf.lower_object_size)
+        self.upper_object_size = get_size_bytes(conf.upper_object_size)
         self.files = []
         if self.object_sources:
             self.object_sources = self.object_sources.split()
-            self.files = [file(f, 'rb').read() for f in self.object_sources]
+            self.files = []
+            for f in self.object_sources:
+                with open(f, 'rb') as fp:
+                    self.files.append(fp.read())
 
         self.put_concurrency = int(conf.put_concurrency)
         self.get_concurrency = int(conf.get_concurrency)
@@ -270,7 +281,7 @@ class Bench(object):
         self.heartbeat -= 13    # just to get the first report quicker
         self.failures = 0
         self.complete = 0
-        for i in xrange(self.total):
+        for i in range(self.total):
             if self.aborted:
                 break
             pool.spawn_n(self._run, i)
@@ -311,7 +322,7 @@ class DistributedBenchController(object):
         self.logger = logger
         # ... INFO 1000 PUTS **FINAL** [0 failures], 34.9/s
         self.final_re = re.compile(
-            'INFO (\d+) (.*) \*\*FINAL\*\* \[(\d+) failures\], (\d+\.\d+)/s')
+            r'INFO (\d+) (.*) \*\*FINAL\*\* \[(\d+) failures\], (\d+\.\d+)/s')
         self.clients = conf.bench_clients
         del conf.bench_clients
         for key, minval in [('put_concurrency', 1),
@@ -335,7 +346,7 @@ class DistributedBenchController(object):
             'DEL': dict(count=0, failures=0, rate=0.0),
         }
         for result in pile:
-            for k, v in result.iteritems():
+            for k, v in result.items():
                 target = results[k]
                 target['count'] += int(v['count'])
                 target['failures'] += int(v['failures'])
@@ -349,11 +360,12 @@ class DistributedBenchController(object):
         s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
         ip, port = client.split(':')
         s.connect((ip, int(port)))
-        s.sendall(json.dumps(self.conf.__dict__))
+        s.sendall(json.dumps(self.conf.__dict__).encode('ascii'))
         s.shutdown(socket.SHUT_WR)
         s_file = s.makefile('rb', 1)
         result = {}
         for line in s_file:
+            line = line.decode('ascii')
             match = self.final_re.search(line)
             if match:
                 g = match.groups()
@@ -380,9 +392,10 @@ class BenchController(object):
 
     def sigint1(self, signum, frame):
         if self.delete:
-            print >>sys.stderr, (
+            print(
                 'SIGINT received; finishing up and running DELETE.\n'
-                'Send one more SIGINT to exit *immediately*.')
+                'Send one more SIGINT to exit *immediately*.',
+                file=sys.stderr)
             self.aborted = True
             if self.running and not isinstance(self.running, BenchDELETE):
                 self.running.aborted = True
@@ -437,7 +450,8 @@ class BenchDELETE(Bench):
                     direct_client.direct_delete_object(node, partition,
                                                        self.account,
                                                        container_name, name)
-            except client.ClientException as e:
+            except (client.ClientException,
+                    requests.exceptions.ConnectionError) as e:
                 self.logger.debug(str(e))
                 self.failures += 1
         self.complete += 1
@@ -459,14 +473,20 @@ class BenchGET(Bench):
         with self.connection() as conn:
             try:
                 if self.use_proxy:
-                    client.get_object(self.url, self.token,
-                                      container_name, name, http_conn=conn)
+                    headers, body = client.get_object(
+                        self.url, self.token,
+                        container_name, name, http_conn=conn,
+                        resp_chunk_size=2**20)
+                    with closing(body):
+                        for _ in body:
+                            pass
                 else:
                     node = {'ip': self.ip, 'port': self.port, 'device': device}
                     direct_client.direct_get_object(node, partition,
                                                     self.account,
                                                     container_name, name)
-            except client.ClientException as e:
+            except (client.ClientException,
+                    requests.exceptions.ConnectionError) as e:
                 self.logger.debug(str(e))
                 self.failures += 1
         self.complete += 1
@@ -510,7 +530,8 @@ class BenchPUT(Bench):
                                                     container_name, name,
                                                     source,
                                                     content_length=len(source))
-            except client.ClientException as e:
+            except (client.ClientException,
+                    requests.exceptions.ConnectionError) as e:
                 self.logger.debug(str(e))
                 self.failures += 1
             else:
